import json

import pytest

import stripe
from stripe._webhook import Webhook
from stripe.v2.core._event import EventNotification

BINARY_ENCODERS = [
    lambda p: p.encode("utf-8"),
    lambda p: bytearray(p, "utf-8"),
]


@pytest.fixture
def client():
    return stripe.StripeClient("sk_test_fake")


@pytest.fixture
def eventbridge_payload():
    return json.dumps(
        {
            "version": "0",
            "id": "17e8dff5-d6cd-3770-ace9-aeac02b6ac3f",
            "detail-type": "customer.created",
            "source": "aws.partner/stripe.com/ed_123",
            "account": "506417113029",
            "time": "2024-03-07T18:27:56Z",
            "region": "us-west-2",
            "resources": [],
            "detail": {
                "id": "evt_test_123",
                "object": "event",
                "api_version": "2023-10-16",
                "created": 1709836076,
                "data": {"object": {"id": "cus_123", "object": "customer"}},
                "livemode": True,
                "pending_webhooks": 0,
                "request": {"id": "req_123", "idempotency_key": None},
                "type": "customer.created",
            },
        }
    )


@pytest.fixture
def eventgrid_payload():
    return json.dumps(
        {
            "specversion": "1.0",
            "type": "customer.created",
            "source": "/providers/stripe/ed_test_123",
            "id": "9aeb0fdf-c01e-0131-0922-9eb54906e209",
            "time": "2025-07-11T14:30:00Z",
            "subject": None,
            "dataContentType": "application/cloudevents+json",
            "data": {
                "id": "evt_test_456",
                "object": "event",
                "api_version": "2023-10-16",
                "created": 1709836076,
                "data": {"object": {"id": "cus_456", "object": "customer"}},
                "livemode": False,
                "pending_webhooks": 0,
                "request": {"id": "req_456", "idempotency_key": None},
                "type": "customer.created",
            },
        }
    )


@pytest.fixture
def eventbridge_notification_payload():
    return json.dumps(
        {
            "version": "0",
            "id": "17e8dff5-d6cd-3770-ace9-aeac02b6ac3f",
            "detail-type": "v2.core.event_destination.ping",
            "source": "aws.partner/stripe.com/ed_123",
            "account": "506417113029",
            "time": "2024-03-07T18:27:56Z",
            "region": "us-west-2",
            "resources": [],
            "detail": {
                "id": "evt_test_789",
                "object": "v2.core.event",
                "type": "v2.core.event_destination.ping",
                "created": "2024-03-07T18:27:56.000Z",
                "context": "acct_123",
                "livemode": True,
                "related_object": {
                    "id": "ed_123",
                    "type": "v2.core.event_destination",
                    "url": "/v2/core/event_destinations/ed_123",
                },
            },
        }
    )


@pytest.fixture
def eventgrid_notification_payload():
    return json.dumps(
        {
            "specversion": "1.0",
            "type": "v2.core.event_destination.ping",
            "source": "/providers/stripe/ed_test_123",
            "id": "9aeb0fdf-c01e-0131-0922-9eb54906e209",
            "time": "2025-07-11T14:30:00Z",
            "data": {
                "id": "evt_test_790",
                "object": "v2.core.event",
                "type": "v2.core.event_destination.ping",
                "created": "2024-03-07T18:27:56.000Z",
                "context": "acct_123",
                "livemode": True,
                "related_object": {
                    "id": "ed_test_123",
                    "type": "v2.core.event_destination",
                    "url": "/v2/core/event_destinations/ed_test_123",
                },
            },
        }
    )


class TestConstructEventWithoutVerification:
    def test_eventbridge(self, client, eventbridge_payload):
        result = client.construct_event_without_verification(
            eventbridge_payload
        )
        assert isinstance(result, stripe.Event)
        assert result.id == "evt_test_123"
        assert result.type == "customer.created"

    def test_eventgrid(self, client, eventgrid_payload):
        result = client.construct_event_without_verification(eventgrid_payload)
        assert isinstance(result, stripe.Event)
        assert result.id == "evt_test_456"
        assert result.type == "customer.created"

    def test_raw_event_passthrough(self, client):
        raw_event = json.dumps(
            {
                "id": "evt_test_123",
                "object": "event",
                "type": "customer.created",
            }
        )
        result = client.construct_event_without_verification(raw_event)
        assert isinstance(result, stripe.Event)
        assert result.id == "evt_test_123"
        assert result.type == "customer.created"

    def test_invalid_json(self, client):
        with pytest.raises(json.JSONDecodeError):
            client.construct_event_without_verification("not valid json")

    def test_thin_event_suggests_parse_event_notification_without_verification(
        self, client, eventbridge_notification_payload
    ):
        with pytest.raises(ValueError, match="parse_event_notification"):
            client.construct_event_without_verification(
                eventbridge_notification_payload
            )

    def test_unrecognized_format(self, client):
        with pytest.raises(ValueError, match="Unrecognized event format"):
            client.construct_event_without_verification(
                json.dumps({"foo": "bar"})
            )

    def test_azure_envelope_missing_data_field(self, client):
        payload = json.dumps(
            {
                "specversion": "1.0",
                "type": "customer.created",
                "source": "/providers/stripe/ed_test_123",
                "id": "test-missing-data",
            }
        )
        with pytest.raises(ValueError, match="Unrecognized event format"):
            client.construct_event_without_verification(payload)

    @pytest.mark.parametrize(
        "payload_fixture", ["eventbridge_payload", "eventgrid_payload"]
    )
    @pytest.mark.parametrize(
        "encode", BINARY_ENCODERS, ids=["bytes", "bytearray"]
    )
    def test_binary_envelope(self, request, client, payload_fixture, encode):
        """An envelope can arrive as bytes or a bytearray, which is how many frameworks expose it"""
        payload = request.getfixturevalue(payload_fixture)

        result = client.construct_event_without_verification(encode(payload))

        assert isinstance(result, stripe.Event)
        assert result.type == "customer.created"

    def test_webhook_static_method_eventbridge(self, eventbridge_payload):
        result = Webhook.construct_event_without_verification(
            eventbridge_payload
        )
        assert isinstance(result, stripe.Event)
        assert result.id == "evt_test_123"
        assert result.type == "customer.created"


class TestParseEventNotificationWithoutVerification:
    def test_eventbridge(self, client, eventbridge_notification_payload):
        result = client.parse_event_notification_without_verification(
            eventbridge_notification_payload
        )
        assert result.id == "evt_test_789"
        assert result.type == "v2.core.event_destination.ping"

    def test_eventgrid(self, client, eventgrid_notification_payload):
        result = client.parse_event_notification_without_verification(
            eventgrid_notification_payload
        )
        assert result.id == "evt_test_790"
        assert result.type == "v2.core.event_destination.ping"

    @pytest.mark.parametrize(
        "payload_fixture",
        [
            "eventbridge_notification_payload",
            "eventgrid_notification_payload",
        ],
    )
    @pytest.mark.parametrize(
        "encode", BINARY_ENCODERS, ids=["bytes", "bytearray"]
    )
    def test_binary_envelope(self, request, client, payload_fixture, encode):
        """An envelope can arrive as bytes or a bytearray, which is how many frameworks expose it"""
        payload = request.getfixturevalue(payload_fixture)

        result = client.parse_event_notification_without_verification(
            encode(payload)
        )

        assert isinstance(result, EventNotification)
        assert result.type == "v2.core.event_destination.ping"

    def test_v1_event_suggests_construct_event_without_verification(
        self, client, eventbridge_payload
    ):
        with pytest.raises(ValueError, match="construct_event"):
            client.parse_event_notification_without_verification(
                eventbridge_payload
            )

    def test_invalid_json(self, client):
        with pytest.raises(json.JSONDecodeError):
            client.parse_event_notification_without_verification(
                "not valid json"
            )

    def test_unrecognized_format(self, client):
        with pytest.raises(ValueError, match="Unrecognized event format"):
            client.parse_event_notification_without_verification(
                json.dumps({"foo": "bar"})
            )

    def test_azure_envelope_missing_data_field(self, client):
        payload = json.dumps(
            {
                "specversion": "1.0",
                "type": "v2.core.event_destination.ping",
                "source": "/providers/stripe/ed_test_123",
                "id": "test-missing-data",
            }
        )
        with pytest.raises(ValueError, match="Unrecognized event format"):
            client.parse_event_notification_without_verification(payload)

    def test_unexpected_object_type_in_event_notification(self, client):
        # Wrap in an EventBridge envelope so envelope extraction succeeds,
        # letting the object-type guard inside from_json fire.
        payload = json.dumps(
            {
                "version": "0",
                "id": "17e8dff5-d6cd-3770-ace9-aeac02b6ac3f",
                "detail-type": "customer.created",
                "source": "aws.partner/stripe.com/ed_123",
                "account": "506417113029",
                "time": "2024-03-07T18:27:56Z",
                "region": "us-west-2",
                "resources": [],
                "detail": {
                    "object": "customer",
                    "type": "customer.created",
                    "id": "cus_123",
                },
            }
        )
        with pytest.raises(ValueError, match="Unexpected object type"):
            client.parse_event_notification_without_verification(payload)

    def test_raw_event_notification_passthrough(self, client):
        raw_notification = json.dumps(
            {
                "id": "evt_234",
                "object": "v2.core.event",
                "type": "v2.core.event_destination.ping",
                "created": "2022-02-15T00:27:45.330Z",
                "livemode": True,
                "context": "acct_123",
                "related_object": {
                    "id": "ed_123",
                    "type": "v2.core.event_destination",
                    "url": "/v2/core/event_destinations/ed_123",
                },
            }
        )
        result = client.parse_event_notification_without_verification(
            raw_notification
        )
        assert isinstance(result, EventNotification)
        assert result.id == "evt_234"
        assert result.type == "v2.core.event_destination.ping"



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Union
from typing_extensions import Literal


class MeterEventAdjustment(StripeObject):
    """
    A Meter Event Adjustment is used to cancel or modify previously recorded meter events. Meter Event Adjustments allow you to correct billing data by canceling individual events or event ranges, with tracking of adjustment status and creation time.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.meter_event_adjustment"]] = (
        "v2.billing.meter_event_adjustment"
    )

    class Cancel(StripeObject):
        identifier: str
        """
        The identifier that was originally assigned to the meter event. You can only cancel events within 24 hours of Stripe receiving them.
        """

    cancel: Cancel
    """
    Specifies which event to cancel.
    """
    created: str
    """
    The time the adjustment was created.
    """
    event_name: str
    """
    The name of the meter event. Corresponds with the `event_name` field on a meter.
    """
    id: str
    """
    The unique ID of this meter event adjustment.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.meter_event_adjustment"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Union[Literal["complete", "pending"], str]
    """
    Open Enum. The meter event adjustment's status.
    """
    type: Literal["cancel"]
    """
    Open Enum. Specifies the type of cancellation. Currently supports canceling a single event.
    """
    _inner_class_types = {"cancel": Cancel}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.checkout._session import Session
    from stripe.checkout._session_line_item_service import (
        SessionLineItemService,
    )
    from stripe.params.checkout._session_create_params import (
        SessionCreateParams,
    )
    from stripe.params.checkout._session_expire_params import (
        SessionExpireParams,
    )
    from stripe.params.checkout._session_list_params import SessionListParams
    from stripe.params.checkout._session_retrieve_params import (
        SessionRetrieveParams,
    )
    from stripe.params.checkout._session_update_params import (
        SessionUpdateParams,
    )

_subservices = {
    "line_items": [
        "stripe.checkout._session_line_item_service",
        "SessionLineItemService",
    ],
}


class SessionService(StripeService):
    line_items: "SessionLineItemService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["SessionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Session]":
        """
        Returns a list of Checkout Sessions.
        """
        return cast(
            "ListObject[Session]",
            self._request(
                "get",
                "/v1/checkout/sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["SessionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Session]":
        """
        Returns a list of Checkout Sessions.
        """
        return cast(
            "ListObject[Session]",
            await self._request_async(
                "get",
                "/v1/checkout/sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["SessionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Creates a Checkout Session object.
        """
        return cast(
            "Session",
            self._request(
                "post",
                "/v1/checkout/sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["SessionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Creates a Checkout Session object.
        """
        return cast(
            "Session",
            await self._request_async(
                "post",
                "/v1/checkout/sessions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        session: str,
        params: Optional["SessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Retrieves a Checkout Session object.
        """
        return cast(
            "Session",
            self._request(
                "get",
                "/v1/checkout/sessions/{session}".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        session: str,
        params: Optional["SessionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Retrieves a Checkout Session object.
        """
        return cast(
            "Session",
            await self._request_async(
                "get",
                "/v1/checkout/sessions/{session}".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        session: str,
        params: Optional["SessionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Updates a Checkout Session object.

        Related guide: [Dynamically update a Checkout Session](https://docs.stripe.com/payments/advanced/dynamic-updates)
        """
        return cast(
            "Session",
            self._request(
                "post",
                "/v1/checkout/sessions/{session}".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        session: str,
        params: Optional["SessionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        Updates a Checkout Session object.

        Related guide: [Dynamically update a Checkout Session](https://docs.stripe.com/payments/advanced/dynamic-updates)
        """
        return cast(
            "Session",
            await self._request_async(
                "post",
                "/v1/checkout/sessions/{session}".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def expire(
        self,
        session: str,
        params: Optional["SessionExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        A Checkout Session can be expired when it is in one of these statuses: open

        After it expires, a customer can't complete a Checkout Session and customers loading the Checkout Session see a message saying the Checkout Session is expired.
        """
        return cast(
            "Session",
            self._request(
                "post",
                "/v1/checkout/sessions/{session}/expire".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def expire_async(
        self,
        session: str,
        params: Optional["SessionExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Session":
        """
        A Checkout Session can be expired when it is in one of these statuses: open

        After it expires, a customer can't complete a Checkout Session and customers loading the Checkout Session see a message saying the Checkout Session is expired.
        """
        return cast(
            "Session",
            await self._request_async(
                "post",
                "/v1/checkout/sessions/{session}/expire".format(
                    session=sanitize_id(session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



from stripe._request_options import (
    RequestOptions,
    merge_options,
    extract_options_from_dict,
)
from stripe._requestor_options import RequestorOptions


class TestRequestOptions(object):
    def test_merge(self):
        options = RequestorOptions(
            api_key="sk_test_123",
            stripe_account="acct_123",
            base_addresses={"api": "https://api.stripe.com"},
        )
        other: RequestOptions = {
            "api_key": "sk_test_456",
            "stripe_version": "2020-01-01",
            "headers": {"foo": "bar"},
        }
        merged = merge_options(options, other)
        assert merged.get("api_key") == "sk_test_456"
        assert merged.get("stripe_version") == "2020-01-01"
        assert merged.get("stripe_account") == "acct_123"
        assert merged.get("headers") == {"foo": "bar"}

    def test_merge_none(self):
        options = RequestorOptions(
            api_key="sk_test_123",
            stripe_account="acct_123",
            base_addresses={"api": "https://api.stripe.com"},
        )
        merged = merge_options(options, None)
        assert merged.get("api_key") == "sk_test_123"
        assert merged.get("stripe_version") is None
        assert merged.get("stripe_account") == "acct_123"
        assert merged.get("headers") is None

    def test_extract_from_dict(self):
        options, remaining = extract_options_from_dict(
            {
                "api_key": "sk_test_123",
                "stripe_version": "2020-01-01",
                "stripe_account": "acct_123",
                "stripe_context": "wksp_123",
                "idempotency_key": "idemp_123",
                "headers": {
                    "X-Stripe-Header": "Some-Value",
                },
                "foo": "bar",
            }
        )
        assert options.get("api_key") == "sk_test_123"
        assert options.get("stripe_version") == "2020-01-01"
        assert options.get("stripe_account") == "acct_123"
        assert options.get("stripe_context") == "wksp_123"
        assert options.get("idempotency_key") == "idemp_123"
        assert options.get("headers") == {"X-Stripe-Header": "Some-Value"}
        assert remaining == {"foo": "bar"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional, Union
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._charge import Charge
    from stripe._invoice import Invoice
    from stripe._payment_intent import PaymentIntent
    from stripe._payment_record import PaymentRecord
    from stripe.params._invoice_payment_list_params import (
        InvoicePaymentListParams,
    )
    from stripe.params._invoice_payment_retrieve_params import (
        InvoicePaymentRetrieveParams,
    )


class InvoicePayment(ListableAPIResource["InvoicePayment"]):
    """
    Invoice Payments represent payments made against invoices. Invoice Payments can
    be accessed in two ways:
    1. By expanding the `payments` field on the [Invoice](https://api.stripe.com#invoice) resource.
    2. By using the Invoice Payment retrieve and list endpoints.

    Invoice Payments include the mapping between payment objects, such as Payment Intent, and Invoices.
    This resource and its endpoints allows you to easily track if a payment is associated with a specific invoice and
    monitor the allocation details of the payments.
    """

    OBJECT_NAME: ClassVar[Literal["invoice_payment"]] = "invoice_payment"

    class Payment(StripeObject):
        charge: Optional[ExpandableField["Charge"]]
        """
        ID of the successful charge for this payment when `type` is `charge`.Note: charge is only surfaced if the charge object is not associated with a payment intent. If the charge object does have a payment intent, the Invoice Payment surfaces the payment intent instead.
        """
        payment_intent: Optional[ExpandableField["PaymentIntent"]]
        """
        ID of the PaymentIntent associated with this payment when `type` is `payment_intent`. Note: This property is only populated for invoices finalized on or after March 15th, 2019.
        """
        payment_record: Optional[ExpandableField["PaymentRecord"]]
        """
        ID of the PaymentRecord associated with this payment when `type` is `payment_record`.
        """
        type: Union[Literal["charge", "payment_intent", "payment_record"], str]
        """
        Type of payment object associated with this invoice payment.
        """

    class StatusTransitions(StripeObject):
        canceled_at: Optional[int]
        """
        The time that the payment was canceled.
        """
        paid_at: Optional[int]
        """
        The time that the payment succeeded.
        """

    amount_paid: Optional[int]
    """
    Amount that was actually paid for this invoice, in cents (or local equivalent). This field is null until the payment is `paid`. This amount can be less than the `amount_requested` if the PaymentIntent's `amount_received` is not sufficient to pay all of the invoices that it is attached to.
    """
    amount_requested: int
    """
    Amount intended to be paid toward this invoice, in cents (or local equivalent)
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    id: str
    """
    Unique identifier for the object.
    """
    invoice: ExpandableField["Invoice"]
    """
    The invoice that was paid.
    """
    is_default: bool
    """
    Stripe automatically creates a default InvoicePayment when the invoice is finalized, and keeps it synchronized with the invoice's `amount_remaining`. The PaymentIntent associated with the default payment can't be edited or canceled directly.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["invoice_payment"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payment: Payment
    status: str
    """
    The status of the payment, one of `open`, `paid`, or `canceled`.
    """
    status_transitions: StatusTransitions

    @classmethod
    def list(
        cls, **params: Unpack["InvoicePaymentListParams"]
    ) -> ListObject["InvoicePayment"]:
        """
        When retrieving an invoice, there is an includable payments property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of payments.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["InvoicePaymentListParams"]
    ) -> ListObject["InvoicePayment"]:
        """
        When retrieving an invoice, there is an includable payments property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of payments.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["InvoicePaymentRetrieveParams"]
    ) -> "InvoicePayment":
        """
        Retrieves the invoice payment with the given ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["InvoicePaymentRetrieveParams"]
    ) -> "InvoicePayment":
        """
        Retrieves the invoice payment with the given ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "payment": Payment,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-

import json
from typing import Any, ClassVar, Dict, Optional, cast, Union
# v2-event-imports: The beginning of the section generated from our OpenAPI spec
# v2-event-imports: The end of the section generated from our OpenAPI spec

from typing_extensions import Literal, TYPE_CHECKING

from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._util import get_api_mode
from stripe._stripe_context import StripeContext
from stripe._webhook import WebhookPayload

if TYPE_CHECKING:
    from stripe._stripe_client import StripeClient


# The beginning of the section generated from our OpenAPI spec
class Event(StripeObject):
    """
    Events are generated to keep you informed of activity in your business account. APIs in the /v2 namespace generate [thin events](https://docs.stripe.com/event-destinations#benefits-of-thin-events) which have small, unversioned payloads that include a reference to the ID of the object that has changed. The Events v2 API returns these new thin events. [Retrieve the event object](https://docs.stripe.com/event-destinations#fetch-data) for additional data about the event. Use the related object ID in the event payload to [fetch the API resource](https://docs.stripe.com/event-destinations#retrieve-the-object-associated-with-thin-events) of the object associated with the event. Comparatively, events generated by most API v1 include a versioned snapshot of an API object in their payload.
    You can access events through the [Retrieve Event API](https://docs.stripe.com/api/v2/core/events/retrieve) for 30 days.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.event"]] = "v2.core.event"

    class Reason(StripeObject):
        class Request(StripeObject):
            id: str
            """
            ID of the API request that caused the event.
            """
            idempotency_key: str
            """
            The idempotency key transmitted during the request.
            """

        request: Optional[Request]
        """
        Information on the API request that instigated the event.
        """
        type: Literal["request"]
        """
        Event reason type.
        """
        _inner_class_types = {"request": Request}

    changes: Optional[UntypedStripeObject[Any]]
    """
    Before and after changes for the primary related object.
    """
    context: Optional[str]
    """
    Authentication context needed to fetch the event or related object.
    """
    created: str
    """
    Time at which the object was created.
    """
    id: str
    """
    Unique identifier for the event.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.event"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    reason: Optional[Reason]
    """
    Reason for the event.
    """
    type: str
    """
    The type of the event.
    """
    _inner_class_types = {"reason": Reason}


# The end of the section generated from our OpenAPI spec


class ReasonRequest:
    id: str
    idempotency_key: str

    def __init__(self, d) -> None:
        self.id = d["id"]
        self.idempotency_key = d["idempotency_key"]

    def __repr__(self) -> str:
        return f"<ReasonRequest id={self.id} idempotency_key={self.idempotency_key}>"


class Reason:
    type: Literal["request"]
    request: Optional[ReasonRequest] = None

    def __init__(self, d) -> None:
        self.type = d["type"]
        if self.type == "request":
            self.request = ReasonRequest(d["request"])

    def __repr__(self) -> str:
        return f"<Reason type={self.type} request={self.request}>"


class RelatedObject:
    id: str
    type: str
    url: str

    def __init__(self, d) -> None:
        self.id = d["id"]
        self.type = d["type"]
        self.url = d["url"]

    def __repr__(self) -> str:
        return f"<RelatedObject id={self.id} type={self.type} url={self.url}>"


class EventNotification:
    """
    EventNotification represents the json that's delivered from an Event Destination. It's a basic struct-like object with a few convenience methods. Use `fetch_event()` to get the full event object.
    """

    id: str
    """
    Unique identifier for the event.
    """
    type: str
    """
    The type of the event.
    """
    created: str
    """
    Time at which the object was created.
    """
    livemode: bool
    """
    Livemode indicates if the event is from a production(true) or test(false) account.
    """
    object: str
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    context: Optional[StripeContext] = None
    """
    [Optional] Authentication context needed to fetch the event or related object.
    """
    reason: Optional[Reason] = None
    """
    [Optional] Reason for the event.
    """

    def __init__(
        self, parsed_body: Dict[str, Any], client: "StripeClient"
    ) -> None:
        self.id = parsed_body["id"]
        self.object = parsed_body["object"]
        self.type = parsed_body["type"]
        self.created = parsed_body["created"]
        self.livemode = bool(parsed_body.get("livemode"))
        context_value = parsed_body.get("context")
        if context_value:
            self.context = StripeContext.parse(context_value)

        if parsed_body.get("reason"):
            self.reason = Reason(parsed_body["reason"])

        self._client = client

    @staticmethod
    def from_json(
        payload: Union[WebhookPayload, Dict[str, Any]], client: "StripeClient"
    ) -> "EventNotification":
        """
        Helper for constructing an Event Notification. Doesn't perform signature validation, so you
        should use StripeClient.parse_event_notification() instead for initial handling.
        This is useful in unit tests and working with EventNotifications whose authenticity you've already validated.
        """
        if isinstance(payload, dict):
            parsed_body = payload
        else:
            parsed_body = json.loads(
                payload.decode("utf-8")
                if isinstance(payload, (bytes, bytearray))
                else payload
            )

        if parsed_body.get("object") == "event":
            raise ValueError(
                "You passed a webhook payload to StripeClient.parse_event_notification, which expects a thin event notification. Use StripeClient.construct_event instead."
            )
        if (
            parsed_body.get("object") is not None
            and parsed_body.get("object") != "v2.core.event"
        ):
            raise ValueError(
                f"Unexpected object type '{parsed_body.get('object')}'. Expected 'v2.core.event' for an event notification."
            )

        # circular import busting
        from stripe.events._event_classes import (
            get_v2_event_notification_class,
        )

        event_class = get_v2_event_notification_class(parsed_body["type"])

        return event_class(parsed_body, client)

    def __repr__(self) -> str:
        return f"<EventNotification id={self.id} type={self.type} created={self.created} context={self.context} reason={self.reason}>"

    def fetch_event(self) -> Event:
        response = self._client.raw_request(
            "get",
            f"/v2/core/events/{self.id}",
            stripe_context=self.context,
            headers={"Stripe-Request-Trigger": f"event={self.id}"},
            usage=["pushed_event_pull"],
        )
        return cast(Event, self._client.deserialize(response, api_mode="V2"))

    async def fetch_event_async(self) -> Event:
        response = await self._client.raw_request_async(
            "get",
            f"/v2/core/events/{self.id}",
            stripe_context=self.context,
            headers={"Stripe-Request-Trigger": f"event={self.id}"},
            usage=["pushed_event_pull", "pushed_event_pull_async"],
        )
        return cast(Event, self._client.deserialize(response, api_mode="V2"))


class UnknownEventNotification(EventNotification):
    """
    Represents an EventNotification payload that the SDK doesn't have types for. May have a related object.
    """

    related_object: Optional[RelatedObject] = None
    """
    [Optional] Object containing the reference to API resource relevant to the event.
    """

    def __init__(
        self, parsed_body: Dict[str, Any], client: "StripeClient"
    ) -> None:
        super().__init__(parsed_body, client)

        if parsed_body.get("related_object"):
            self.related_object = RelatedObject(parsed_body["related_object"])

    def fetch_related_object(self) -> Optional[StripeObject]:
        if self.related_object is None:
            return None

        response = self._client.raw_request(
            "get",
            self.related_object.url,
            stripe_context=self.context,
            headers={"Stripe-Request-Trigger": f"event={self.id}"},
            usage=["fetch_related_object", "unknown_event"],
        )
        return self._client.deserialize(
            response,
            api_mode=get_api_mode(self.related_object.url),
        )

    async def fetch_related_object_async(self) -> Optional[StripeObject]:
        if self.related_object is None:
            return None

        response = await self._client.raw_request_async(
            "get",
            self.related_object.url,
            stripe_context=self.context,
            headers={"Stripe-Request-Trigger": f"event={self.id}"},
            usage=["fetch_related_object", "unknown_event"],
        )
        return self._client.deserialize(
            response,
            api_mode=get_api_mode(self.related_object.url),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class SecretDeleteWhereParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    name: str
    """
    A name for the secret that's unique within the scope.
    """
    scope: "SecretDeleteWhereParamsScope"
    """
    Specifies the scoping of the secret. Requests originating from UI extensions can only access account-scoped secrets or secrets scoped to their own user.
    """


class SecretDeleteWhereParamsScope(TypedDict):
    type: Union[Literal["account", "user"], str]
    """
    The secret scope type.
    """
    user: NotRequired[str]
    """
    The user ID. This field is required if `type` is set to `user`, and should not be provided if `type` is set to `account`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional, Union
from typing_extensions import Literal


class ProductCatalogImport(StripeObject):
    """
    The ProductCatalogImport object tracks the long-running background process that handles uploading, processing and validation.
    """

    OBJECT_NAME: ClassVar[Literal["v2.commerce.product_catalog_import"]] = (
        "v2.commerce.product_catalog_import"
    )

    class StatusDetails(StripeObject):
        class AwaitingUpload(StripeObject):
            class UploadUrl(StripeObject):
                expires_at: str
                """
                The timestamp when the upload URL expires.
                """
                url: str
                """
                The pre-signed URL for uploading the catalog file.
                """

            upload_url: UploadUrl
            """
            The pre-signed URL information for uploading the catalog file.
            """
            _inner_class_types = {"upload_url": UploadUrl}

        class Failed(StripeObject):
            code: Union[
                Literal["file_not_found", "internal_error", "invalid_file"],
                str,
            ]
            """
            The error code for this product catalog processing failure.
            """
            failure_message: str
            """
            A message explaining why the import failed.
            """
            type: Union[Literal["cannot_proceed", "transient_failure"], str]
            """
            The error type for this product catalog processing failure.
            """

        class Processing(StripeObject):
            error_count: int
            """
            The number of records that failed to process so far.
            """
            success_count: int
            """
            The number of records processed so far.
            """
            _field_encodings = {
                "error_count": "int64_string",
                "success_count": "int64_string",
            }

        class Succeeded(StripeObject):
            success_count: int
            """
            The total number of records processed.
            """
            _field_encodings = {"success_count": "int64_string"}

        class SucceededWithErrors(StripeObject):
            class ErrorFile(StripeObject):
                class DownloadUrl(StripeObject):
                    expires_at: str
                    """
                    The timestamp when the download URL expires.
                    """
                    url: str
                    """
                    The pre-signed URL for downloading the error file.
                    """

                content_type: str
                """
                The MIME type of the error file.
                """
                download_url: DownloadUrl
                """
                The pre-signed URL information for downloading the error file.
                """
                size: int
                """
                The size of the error file in bytes.
                """
                _inner_class_types = {"download_url": DownloadUrl}
                _field_encodings = {"size": "int64_string"}

            class Sample(StripeObject):
                error_message: str
                """
                A description of what went wrong with this record.
                """
                field: str
                """
                The name of the field that caused the error.
                """
                id: str
                """
                The identifier of the record that failed to process.
                """
                row: int
                """
                The row number in the import file where the error occurred.
                """
                _field_encodings = {"row": "int64_string"}

            error_count: int
            """
            The total number of records that failed to process.
            """
            error_file: ErrorFile
            """
            A file containing details about all errors that occurred.
            """
            samples: List[Sample]
            """
            A sample of errors that occurred during processing.
            """
            success_count: int
            """
            The total number of records processed.
            """
            _inner_class_types = {"error_file": ErrorFile, "samples": Sample}
            _field_encodings = {
                "error_count": "int64_string",
                "success_count": "int64_string",
            }

        awaiting_upload: Optional[AwaitingUpload]
        """
        Details when the import is awaiting file upload.
        """
        failed: Optional[Failed]
        """
        Details when the import didn't complete.
        """
        processing: Optional[Processing]
        """
        Details when the import is processing.
        """
        succeeded: Optional[Succeeded]
        """
        Details when the import has succeeded.
        """
        succeeded_with_errors: Optional[SucceededWithErrors]
        """
        Details when the import completed but some records failed to process.
        """
        _inner_class_types = {
            "awaiting_upload": AwaitingUpload,
            "failed": Failed,
            "processing": Processing,
            "succeeded": Succeeded,
            "succeeded_with_errors": SucceededWithErrors,
        }

    created: str
    """
    The time this ProductCatalogImport was created.
    """
    feed_type: Union[
        Literal["inventory", "pricing", "product", "promotion"], str
    ]
    """
    The type of feed data being imported into the product catalog.
    """
    id: str
    """
    The unique identifier for this ProductCatalogImport.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: UntypedStripeObject[str]
    """
    Additional information about the object in a structured format.
    """
    mode: Union[Literal["replace", "upsert"], str]
    """
    The import strategy for handling existing catalog data.
    """
    object: Literal["v2.commerce.product_catalog_import"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal[
        "awaiting_upload",
        "failed",
        "processing",
        "succeeded",
        "succeeded_with_errors",
    ]
    """
    The current status of this ProductCatalogImport.
    """
    status_details: Optional[StatusDetails]
    """
    Details about the current import status.
    """
    _inner_class_types = {"status_details": StatusDetails}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class PaymentIntentCancelParams(RequestOptions):
    cancellation_reason: NotRequired[
        "Literal['abandoned', 'duplicate', 'fraudulent', 'requested_by_customer']|str"
    ]
    """
    Reason for canceling this PaymentIntent. Possible values are: `duplicate`, `fraudulent`, `requested_by_customer`, or `abandoned`
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.test_helpers.treasury._inbound_transfer_fail_params import (
        InboundTransferFailParams,
    )
    from stripe.params.test_helpers.treasury._inbound_transfer_return_inbound_transfer_params import (
        InboundTransferReturnInboundTransferParams,
    )
    from stripe.params.test_helpers.treasury._inbound_transfer_succeed_params import (
        InboundTransferSucceedParams,
    )
    from stripe.treasury._inbound_transfer import InboundTransfer


class InboundTransferService(StripeService):
    def fail(
        self,
        id: str,
        params: Optional["InboundTransferFailParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Transitions a test mode created InboundTransfer to the failed status. The InboundTransfer must already be in the processing state.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/fail".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def fail_async(
        self,
        id: str,
        params: Optional["InboundTransferFailParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Transitions a test mode created InboundTransfer to the failed status. The InboundTransfer must already be in the processing state.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/fail".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def return_inbound_transfer(
        self,
        id: str,
        params: Optional["InboundTransferReturnInboundTransferParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Marks the test mode InboundTransfer object as returned and links the InboundTransfer to a ReceivedDebit. The InboundTransfer must already be in the succeeded state.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/return".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def return_inbound_transfer_async(
        self,
        id: str,
        params: Optional["InboundTransferReturnInboundTransferParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Marks the test mode InboundTransfer object as returned and links the InboundTransfer to a ReceivedDebit. The InboundTransfer must already be in the succeeded state.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/return".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def succeed(
        self,
        id: str,
        params: Optional["InboundTransferSucceedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Transitions a test mode created InboundTransfer to the succeeded status. The InboundTransfer must already be in the processing state.
        """
        return cast(
            "InboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/succeed".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def succeed_async(
        self,
        id: str,
        params: Optional["InboundTransferSucceedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "InboundTransfer":
        """
        Transitions a test mode created InboundTransfer to the succeeded status. The InboundTransfer must already be in the processing state.
        """
        return cast(
            "InboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/inbound_transfers/{id}/succeed".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._customer import Customer
    from stripe._customer_balance_transaction_service import (
        CustomerBalanceTransactionService,
    )
    from stripe._customer_cash_balance_service import (
        CustomerCashBalanceService,
    )
    from stripe._customer_cash_balance_transaction_service import (
        CustomerCashBalanceTransactionService,
    )
    from stripe._customer_funding_instructions_service import (
        CustomerFundingInstructionsService,
    )
    from stripe._customer_payment_method_service import (
        CustomerPaymentMethodService,
    )
    from stripe._customer_payment_source_service import (
        CustomerPaymentSourceService,
    )
    from stripe._customer_tax_id_service import CustomerTaxIdService
    from stripe._discount import Discount
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._search_result_object import SearchResultObject
    from stripe.params._customer_create_params import CustomerCreateParams
    from stripe.params._customer_delete_discount_params import (
        CustomerDeleteDiscountParams,
    )
    from stripe.params._customer_delete_params import CustomerDeleteParams
    from stripe.params._customer_list_params import CustomerListParams
    from stripe.params._customer_retrieve_params import CustomerRetrieveParams
    from stripe.params._customer_search_params import CustomerSearchParams
    from stripe.params._customer_update_params import CustomerUpdateParams

_subservices = {
    "balance_transactions": [
        "stripe._customer_balance_transaction_service",
        "CustomerBalanceTransactionService",
    ],
    "cash_balance": [
        "stripe._customer_cash_balance_service",
        "CustomerCashBalanceService",
    ],
    "cash_balance_transactions": [
        "stripe._customer_cash_balance_transaction_service",
        "CustomerCashBalanceTransactionService",
    ],
    "funding_instructions": [
        "stripe._customer_funding_instructions_service",
        "CustomerFundingInstructionsService",
    ],
    "payment_methods": [
        "stripe._customer_payment_method_service",
        "CustomerPaymentMethodService",
    ],
    "payment_sources": [
        "stripe._customer_payment_source_service",
        "CustomerPaymentSourceService",
    ],
    "tax_ids": ["stripe._customer_tax_id_service", "CustomerTaxIdService"],
}


class CustomerService(StripeService):
    balance_transactions: "CustomerBalanceTransactionService"
    cash_balance: "CustomerCashBalanceService"
    cash_balance_transactions: "CustomerCashBalanceTransactionService"
    funding_instructions: "CustomerFundingInstructionsService"
    payment_methods: "CustomerPaymentMethodService"
    payment_sources: "CustomerPaymentSourceService"
    tax_ids: "CustomerTaxIdService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def delete(
        self,
        customer: str,
        params: Optional["CustomerDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Permanently deletes a customer. It cannot be undone. Also immediately cancels any active subscriptions on the customer.
        """
        return cast(
            "Customer",
            self._request(
                "delete",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        customer: str,
        params: Optional["CustomerDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Permanently deletes a customer. It cannot be undone. Also immediately cancels any active subscriptions on the customer.
        """
        return cast(
            "Customer",
            await self._request_async(
                "delete",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        customer: str,
        params: Optional["CustomerRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Retrieves a Customer object.
        """
        return cast(
            "Customer",
            self._request(
                "get",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        customer: str,
        params: Optional["CustomerRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Retrieves a Customer object.
        """
        return cast(
            "Customer",
            await self._request_async(
                "get",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        customer: str,
        params: Optional["CustomerUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Updates the specified customer by setting the values of the parameters passed. Any parameters not provided are left unchanged. For example, if you pass the source parameter, that becomes the customer's active source (such as a card) to be used for all charges in the future. When you update a customer to a new valid card source by passing the source parameter: for each of the customer's current subscriptions, if the subscription bills automatically and is in the past_due state, then the latest open invoice for the subscription with automatic collection enabled is retried. This retry doesn't count as an automatic retry, and doesn't affect the next regularly scheduled payment for the invoice. Changing the default_source for a customer doesn't trigger this behavior.

        This request accepts mostly the same arguments as the customer creation call.
        """
        return cast(
            "Customer",
            self._request(
                "post",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        customer: str,
        params: Optional["CustomerUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Updates the specified customer by setting the values of the parameters passed. Any parameters not provided are left unchanged. For example, if you pass the source parameter, that becomes the customer's active source (such as a card) to be used for all charges in the future. When you update a customer to a new valid card source by passing the source parameter: for each of the customer's current subscriptions, if the subscription bills automatically and is in the past_due state, then the latest open invoice for the subscription with automatic collection enabled is retried. This retry doesn't count as an automatic retry, and doesn't affect the next regularly scheduled payment for the invoice. Changing the default_source for a customer doesn't trigger this behavior.

        This request accepts mostly the same arguments as the customer creation call.
        """
        return cast(
            "Customer",
            await self._request_async(
                "post",
                "/v1/customers/{customer}".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def delete_discount(
        self,
        customer: str,
        params: Optional["CustomerDeleteDiscountParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Discount":
        """
        Removes the currently applied discount on a customer.
        """
        return cast(
            "Discount",
            self._request(
                "delete",
                "/v1/customers/{customer}/discount".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_discount_async(
        self,
        customer: str,
        params: Optional["CustomerDeleteDiscountParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Discount":
        """
        Removes the currently applied discount on a customer.
        """
        return cast(
            "Discount",
            await self._request_async(
                "delete",
                "/v1/customers/{customer}/discount".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["CustomerListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Customer]":
        """
        Returns a list of your customers. The customers are returned sorted by creation date, with the most recent customers appearing first.
        """
        return cast(
            "ListObject[Customer]",
            self._request(
                "get",
                "/v1/customers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CustomerListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Customer]":
        """
        Returns a list of your customers. The customers are returned sorted by creation date, with the most recent customers appearing first.
        """
        return cast(
            "ListObject[Customer]",
            await self._request_async(
                "get",
                "/v1/customers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["CustomerCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Creates a new customer object.
        """
        return cast(
            "Customer",
            self._request(
                "post",
                "/v1/customers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["CustomerCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Creates a new customer object.
        """
        return cast(
            "Customer",
            await self._request_async(
                "post",
                "/v1/customers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def search(
        self,
        params: "CustomerSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Customer]":
        """
        Search for customers you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Customer]",
            self._request(
                "get",
                "/v1/customers/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def search_async(
        self,
        params: "CustomerSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Customer]":
        """
        Search for customers you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Customer]",
            await self._request_async(
                "get",
                "/v1/customers/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Optional, Union, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._tax_code import TaxCode
    from stripe.params._shipping_rate_create_params import (
        ShippingRateCreateParams,
    )
    from stripe.params._shipping_rate_list_params import ShippingRateListParams
    from stripe.params._shipping_rate_modify_params import (
        ShippingRateModifyParams,
    )
    from stripe.params._shipping_rate_retrieve_params import (
        ShippingRateRetrieveParams,
    )


class ShippingRate(
    CreateableAPIResource["ShippingRate"],
    ListableAPIResource["ShippingRate"],
    UpdateableAPIResource["ShippingRate"],
):
    """
    Shipping rates describe the price of shipping presented to your customers and
    applied to a purchase. For more information, see [Charge for shipping](https://docs.stripe.com/payments/during-payment/charge-shipping).
    """

    OBJECT_NAME: ClassVar[Literal["shipping_rate"]] = "shipping_rate"

    class DeliveryEstimate(StripeObject):
        class Maximum(StripeObject):
            unit: Union[
                Literal["business_day", "day", "hour", "month", "week"], str
            ]
            """
            A unit of time.
            """
            value: int
            """
            Must be greater than 0.
            """

        class Minimum(StripeObject):
            unit: Union[
                Literal["business_day", "day", "hour", "month", "week"], str
            ]
            """
            A unit of time.
            """
            value: int
            """
            Must be greater than 0.
            """

        maximum: Optional[Maximum]
        """
        The upper bound of the estimated range. If empty, represents no upper bound i.e., infinite.
        """
        minimum: Optional[Minimum]
        """
        The lower bound of the estimated range. If empty, represents no lower bound.
        """
        _inner_class_types = {"maximum": Maximum, "minimum": Minimum}

    class FixedAmount(StripeObject):
        class CurrencyOptions(StripeObject):
            amount: int
            """
            A non-negative integer in cents representing how much to charge.
            """
            tax_behavior: Union[
                Literal["exclusive", "inclusive", "unspecified"], str
            ]
            """
            Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
            """

        amount: int
        """
        A non-negative integer in cents representing how much to charge.
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        currency_options: Optional[UntypedStripeObject[CurrencyOptions]]
        """
        Shipping rates defined in each available currency option. Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
        """
        _inner_class_types = {"currency_options": CurrencyOptions}
        _inner_class_dicts = ["currency_options"]

    active: bool
    """
    Whether the shipping rate can be used for new purchases. Defaults to `true`.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    delivery_estimate: Optional[DeliveryEstimate]
    """
    The estimated range for how long shipping will take, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    display_name: Optional[str]
    """
    The name of the shipping rate, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    fixed_amount: Optional[FixedAmount]
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["shipping_rate"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    tax_behavior: Optional[
        Union[Literal["exclusive", "inclusive", "unspecified"], str]
    ]
    """
    Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
    """
    tax_code: Optional[ExpandableField["TaxCode"]]
    """
    A [tax code](https://docs.stripe.com/tax/tax-categories) ID. The Shipping tax code is `txcd_92010001`.
    """
    type: Literal["fixed_amount"]
    """
    The type of calculation to use on the shipping rate.
    """

    @classmethod
    def create(
        cls, **params: Unpack["ShippingRateCreateParams"]
    ) -> "ShippingRate":
        """
        Creates a new shipping rate object.
        """
        return cast(
            "ShippingRate",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["ShippingRateCreateParams"]
    ) -> "ShippingRate":
        """
        Creates a new shipping rate object.
        """
        return cast(
            "ShippingRate",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["ShippingRateListParams"]
    ) -> ListObject["ShippingRate"]:
        """
        Returns a list of your shipping rates.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ShippingRateListParams"]
    ) -> ListObject["ShippingRate"]:
        """
        Returns a list of your shipping rates.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["ShippingRateModifyParams"]
    ) -> "ShippingRate":
        """
        Updates an existing shipping rate object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "ShippingRate",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["ShippingRateModifyParams"]
    ) -> "ShippingRate":
        """
        Updates an existing shipping rate object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "ShippingRate",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ShippingRateRetrieveParams"]
    ) -> "ShippingRate":
        """
        Returns the shipping rate object with the given ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ShippingRateRetrieveParams"]
    ) -> "ShippingRate":
        """
        Returns the shipping rate object with the given ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "delivery_estimate": DeliveryEstimate,
        "fixed_amount": FixedAmount,
    }



from typing import Optional
from typing_extensions import TYPE_CHECKING
from stripe._util import merge_dicts
from stripe._stripe_object import StripeObject
from stripe._api_mode import ApiMode

if TYPE_CHECKING:
    # errorImports: The beginning of the section generated from our OpenAPI spec
    from stripe._payment_intent import PaymentIntent
    from stripe._payment_method import PaymentMethod
    from stripe._setup_intent import SetupIntent
    from stripe._source import Source
    # errorImports: The end of the section generated from our OpenAPI spec


class ErrorObject(StripeObject):
    # errorAnnotations: The beginning of the section generated from our OpenAPI spec
    advice_code: Optional[str]
    """
    For card errors resulting from a card issuer decline, a short string indicating [how to proceed with an error](https://docs.stripe.com/declines#retrying-issuer-declines) if they provide one.
    """
    charge: Optional[str]
    """
    For card errors, the ID of the failed charge.
    """
    code: Optional[str]
    """
    For some errors that could be handled programmatically, a short string indicating the [error code](https://docs.stripe.com/error-codes) reported.
    """
    decline_code: Optional[str]
    """
    For card errors resulting from a card issuer decline, a short string indicating the [card issuer's reason for the decline](https://docs.stripe.com/declines#issuer-declines) if they provide one.
    """
    doc_url: Optional[str]
    """
    A URL to more information about the [error code](https://docs.stripe.com/error-codes) reported.
    """
    message: Optional[str]
    """
    A human-readable message providing more details about the error. For card errors, these messages can be shown to your users.
    """
    network_advice_code: Optional[str]
    """
    For card errors resulting from a card issuer decline, a 2 digit code which indicates the advice given to merchant by the card network on how to proceed with an error.
    """
    network_decline_code: Optional[str]
    """
    For payments declined by the network, an alphanumeric code which indicates the reason the payment failed.
    """
    param: Optional[str]
    """
    If the error is parameter-specific, the parameter related to the error. For example, you can use this to display a message near the correct form field.
    """
    payment_intent: Optional["PaymentIntent"]
    """
    The PaymentIntent object for errors returned on a request involving a PaymentIntent.
    """
    payment_method: Optional["PaymentMethod"]
    """
    The PaymentMethod object for errors returned on a request involving a PaymentMethod.
    """
    payment_method_type: Optional[str]
    """
    If the error is specific to the type of payment method, the payment method type that had a problem. This field is only populated for invoice-related errors.
    """
    request_log_url: Optional[str]
    """
    A URL to the request log entry in your dashboard.
    """
    setup_intent: Optional["SetupIntent"]
    """
    The SetupIntent object for errors returned on a request involving a SetupIntent.
    """
    source: Optional["Source"]
    """
    The PaymentSource object for errors returned on a request involving a PaymentSource.
    """
    type: str
    """
    The type of error returned. One of `api_error`, `card_error`, `idempotency_error`, or `invalid_request_error`
    """
    user_message: Optional[str]
    """
    The user message associated with the error.
    """
    # errorAnnotations: The end of the section generated from our OpenAPI spec

    def refresh_from(
        self,
        values,
        api_key=None,
        partial=False,
        stripe_version=None,
        stripe_account=None,
        last_response=None,
        *,
        api_mode: ApiMode = "V1",
    ):
        return self._refresh_from(
            values=values,
            partial=partial,
            last_response=last_response,
            requestor=self._requestor._new_requestor_with_options(
                {
                    "api_key": api_key,
                    "stripe_version": stripe_version,
                    "stripe_account": stripe_account,
                }
            ),
            api_mode=api_mode,
        )

    def _refresh_from(
        self,
        *,
        values,
        partial=False,
        last_response=None,
        requestor,
        api_mode: ApiMode,
    ) -> None:
        # Unlike most other API resources, the API will omit attributes in
        # error objects when they have a null value. We manually set default
        # values here to facilitate generic error handling.
        values = merge_dicts(
            {
                # errorDefaults: The beginning of the section generated from our OpenAPI spec
                "advice_code": None,
                "charge": None,
                "code": None,
                "decline_code": None,
                "doc_url": None,
                "message": None,
                "network_advice_code": None,
                "network_decline_code": None,
                "param": None,
                "payment_intent": None,
                "payment_method": None,
                "payment_method_type": None,
                "request_log_url": None,
                "setup_intent": None,
                "source": None,
                "type": None,
                "user_message": None,
                # errorDefaults: The end of the section generated from our OpenAPI spec
            },
            values,
        )
        return super(ErrorObject, self)._refresh_from(
            values=values,
            partial=partial,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )


class OAuthErrorObject(StripeObject):
    def refresh_from(
        self,
        values,
        api_key=None,
        partial=False,
        stripe_version=None,
        stripe_account=None,
        last_response=None,
        *,
        api_mode: ApiMode = "V1",
    ):
        return self._refresh_from(
            values=values,
            partial=partial,
            last_response=last_response,
            requestor=self._requestor._new_requestor_with_options(
                {
                    "api_key": api_key,
                    "stripe_version": stripe_version,
                    "stripe_account": stripe_account,
                }
            ),
            api_mode=api_mode,
        )

    def _refresh_from(
        self,
        *,
        values,
        partial=False,
        last_response=None,
        requestor,
        api_mode: ApiMode,
    ) -> None:
        # Unlike most other API resources, the API will omit attributes in
        # error objects when they have a null value. We manually set default
        # values here to facilitate generic error handling.
        values = merge_dicts(
            {"error": None, "error_description": None}, values
        )
        return super(OAuthErrorObject, self)._refresh_from(
            values=values,
            partial=partial,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ReaderSetReaderDisplayParams(RequestOptions):
    cart: NotRequired["ReaderSetReaderDisplayParamsCart"]
    """
    Cart details to display on the reader screen, including line items, amounts, and currency.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    type: Literal["cart"]
    """
    Type of information to display. Only `cart` is currently supported.
    """


class ReaderSetReaderDisplayParamsCart(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    line_items: List["ReaderSetReaderDisplayParamsCartLineItem"]
    """
    Array of line items to display.
    """
    tax: NotRequired[int]
    """
    The amount of tax in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    total: int
    """
    Total balance of cart due in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """


class ReaderSetReaderDisplayParamsCartLineItem(TypedDict):
    amount: int
    """
    The price of the item in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    description: str
    """
    The description or name of the item.
    """
    quantity: int
    """
    The quantity of the line item being purchased.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional, Union
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._dispute import Dispute
    from stripe._refund import Refund
    from stripe.reserve._hold import Hold
    from stripe.reserve._plan import Plan


class Release(StripeObject):
    """
    ReserveReleases represent the release of funds from a ReserveHold.
    """

    OBJECT_NAME: ClassVar[Literal["reserve.release"]] = "reserve.release"

    class SourceTransaction(StripeObject):
        dispute: Optional[ExpandableField["Dispute"]]
        """
        The ID of the dispute.
        """
        refund: Optional[ExpandableField["Refund"]]
        """
        The ID of the refund.
        """
        type: Union[Literal["dispute", "refund"], str]
        """
        The type of source transaction.
        """

    amount: int
    """
    Amount released. A positive integer representing how much is released in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal).
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    created_by: Union[Literal["application", "stripe"], str]
    """
    Indicates which party created this ReserveRelease.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["reserve.release"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    reason: Union[
        Literal[
            "bulk_hold_expiry",
            "hold_released_early",
            "hold_reversed",
            "plan_disabled",
        ],
        str,
    ]
    """
    The reason for the ReserveRelease, indicating why the funds were released.
    """
    released_at: int
    """
    The release timestamp of the funds.
    """
    reserve_hold: Optional[ExpandableField["Hold"]]
    """
    The ReserveHold this ReserveRelease is associated with.
    """
    reserve_plan: Optional[ExpandableField["Plan"]]
    """
    The ReservePlan ID this ReserveRelease is associated with. This field is only populated if a ReserveRelease is created by a ReservePlan disable operation, or from a scheduled ReservedHold expiry.
    """
    source_transaction: Optional[SourceTransaction]
    _inner_class_types = {"source_transaction": SourceTransaction}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Optional
from typing_extensions import Literal, NotRequired, TypedDict


class EventDestinationUpdateParams(TypedDict):
    description: NotRequired[str]
    """
    An optional description of what the event destination is used for.
    """
    enabled_events: NotRequired[List[str]]
    """
    The list of events to enable for this endpoint.
    """
    include: NotRequired[List[Literal["webhook_endpoint.url"]]]
    """
    Additional fields to include in the response. Currently supports `webhook_endpoint.url`.
    """
    metadata: NotRequired[
        "Dict[str, Optional[str]]|UntypedStripeObject[Optional[str]]"
    ]
    """
    Metadata.
    """
    name: NotRequired[str]
    """
    Event destination name.
    """
    webhook_endpoint: NotRequired[
        "EventDestinationUpdateParamsWebhookEndpoint"
    ]
    """
    Webhook endpoint configuration.
    """


class EventDestinationUpdateParamsWebhookEndpoint(TypedDict):
    url: str
    """
    The URL of the webhook endpoint.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.billing._feedback_option import FeedbackOption
    from stripe.params.billing._feedback_option_create_params import (
        FeedbackOptionCreateParams,
    )
    from stripe.params.billing._feedback_option_deactivate_params import (
        FeedbackOptionDeactivateParams,
    )
    from stripe.params.billing._feedback_option_list_params import (
        FeedbackOptionListParams,
    )
    from stripe.params.billing._feedback_option_retrieve_params import (
        FeedbackOptionRetrieveParams,
    )
    from stripe.params.billing._feedback_option_update_params import (
        FeedbackOptionUpdateParams,
    )


class FeedbackOptionService(StripeService):
    def list(
        self,
        params: Optional["FeedbackOptionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FeedbackOption]":
        """
        An API method for listing the feedback options model
        """
        return cast(
            "ListObject[FeedbackOption]",
            self._request(
                "get",
                "/v1/billing/feedback_options",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FeedbackOptionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FeedbackOption]":
        """
        An API method for listing the feedback options model
        """
        return cast(
            "ListObject[FeedbackOption]",
            await self._request_async(
                "get",
                "/v1/billing/feedback_options",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "FeedbackOptionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Creates a new feedback option.
        """
        return cast(
            "FeedbackOption",
            self._request(
                "post",
                "/v1/billing/feedback_options",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FeedbackOptionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Creates a new feedback option.
        """
        return cast(
            "FeedbackOption",
            await self._request_async(
                "post",
                "/v1/billing/feedback_options",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["FeedbackOptionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Retrieves a feedback options object given an ID.
        """
        return cast(
            "FeedbackOption",
            self._request(
                "get",
                "/v1/billing/feedback_options/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["FeedbackOptionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Retrieves a feedback options object given an ID.
        """
        return cast(
            "FeedbackOption",
            await self._request_async(
                "get",
                "/v1/billing/feedback_options/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["FeedbackOptionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Updates the description of an existing feedback option.
        """
        return cast(
            "FeedbackOption",
            self._request(
                "post",
                "/v1/billing/feedback_options/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["FeedbackOptionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Updates the description of an existing feedback option.
        """
        return cast(
            "FeedbackOption",
            await self._request_async(
                "post",
                "/v1/billing/feedback_options/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def deactivate(
        self,
        id: str,
        params: Optional["FeedbackOptionDeactivateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Deactivates a feedback option. Deactivated feedback options cannot be used in portal configurations.
        """
        return cast(
            "FeedbackOption",
            self._request(
                "post",
                "/v1/billing/feedback_options/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def deactivate_async(
        self,
        id: str,
        params: Optional["FeedbackOptionDeactivateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FeedbackOption":
        """
        Deactivates a feedback option. Deactivated feedback options cannot be used in portal configurations.
        """
        return cast(
            "FeedbackOption",
            await self._request_async(
                "post",
                "/v1/billing/feedback_options/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._deletable_api_resource import DeletableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._apple_pay_domain_create_params import (
        ApplePayDomainCreateParams,
    )
    from stripe.params._apple_pay_domain_delete_params import (
        ApplePayDomainDeleteParams,
    )
    from stripe.params._apple_pay_domain_list_params import (
        ApplePayDomainListParams,
    )
    from stripe.params._apple_pay_domain_retrieve_params import (
        ApplePayDomainRetrieveParams,
    )


class ApplePayDomain(
    CreateableAPIResource["ApplePayDomain"],
    DeletableAPIResource["ApplePayDomain"],
    ListableAPIResource["ApplePayDomain"],
):
    OBJECT_NAME: ClassVar[Literal["apple_pay_domain"]] = "apple_pay_domain"
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    domain_name: str
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["apple_pay_domain"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def create(
        cls, **params: Unpack["ApplePayDomainCreateParams"]
    ) -> "ApplePayDomain":
        """
        Create an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["ApplePayDomainCreateParams"]
    ) -> "ApplePayDomain":
        """
        Create an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_delete(
        cls, sid: str, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "ApplePayDomain",
            cls._static_request(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def delete(
        sid: str, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        ...

    @overload
    def delete(
        self, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        ...

    @class_method_variant("_cls_delete")
    def delete(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        return self._request_and_refresh(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    async def _cls_delete_async(
        cls, sid: str, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "ApplePayDomain",
            await cls._static_request_async(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def delete_async(
        sid: str, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        ...

    @overload
    async def delete_async(
        self, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        ...

    @class_method_variant("_cls_delete_async")
    async def delete_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ApplePayDomainDeleteParams"]
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        return await self._request_and_refresh_async(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    def list(
        cls, **params: Unpack["ApplePayDomainListParams"]
    ) -> ListObject["ApplePayDomain"]:
        """
        List apple pay domains.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ApplePayDomainListParams"]
    ) -> ListObject["ApplePayDomain"]:
        """
        List apple pay domains.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ApplePayDomainRetrieveParams"]
    ) -> "ApplePayDomain":
        """
        Retrieve an apple pay domain.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ApplePayDomainRetrieveParams"]
    ) -> "ApplePayDomain":
        """
        Retrieve an apple pay domain.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def class_url(cls):
        return "/v1/apple_pay/domains"
